@@ -6,30 +6,35 @@ import aiohttp_cors
 import graphql
 import numpy as np
 from aiohttp import web
+from apischema import schema
 from apischema.graphql import graphql_schema, resolver
 from graphql_server.aiohttp.graphqlview import GraphQLView, _asyncify
+from typing_extensions import Annotated as A
 
-from scanspec.core import Dimension, Path
+from scanspec.core import Frames, Path
 from scanspec.specs import Spec
 
 
 @dataclass
 class Points:
-    """ A collection of singular or multidimensional locations in scan space"""
+    """A collection of singular or multidimensional points in scan space."""
 
     def __init__(self, points: np.ndarray):
         self._points = points
 
     @resolver
     def string(self) -> str:
+        """Truncated string representation of array for debugging."""
         return np.array2string(self._points)
 
     @resolver
     def float_list(self) -> List[float]:
+        """Float list representation of array."""
         return self._points.tolist()
 
     @resolver
     def b64(self) -> str:
+        """Base64 encoded string representation of array."""
         # make sure the data is sent as float64
         assert self._points.dtype == np.dtype(np.float64)
         return base64.b64encode(self._points.tobytes()).decode()
@@ -40,37 +45,33 @@ class Points:
 
 @dataclass
 class AxisFrames:
-    """ A collection of frames (comprising midpoints with lower and upper bounds)
-    present in each axis of the Spec
-    """
+    """The scan points restricted to one particular axis."""
 
-    axis: str
-    """A fixed reference that can be scanned. i.e. a motor, time or
-    number of repetitions.
-    """
-    lower: Points
-    """The lower bounds of each frame (used when fly scanning)"""
-    midpoints: Points
-    """The midpoints of each frame"""
-    upper: Points
-    """The upper bounds of each frame (used when fly scanning)"""
+    axis: A[str, schema(description="An identifier for what to move")]
+    lower: A[
+        Points, schema(description="The lower bounds of each scan frame in each axis"),
+    ]
+    midpoints: A[
+        Points, schema(description="The midpoints of scan frames for each axis")
+    ]
+    upper: A[
+        Points, schema(description="The upper bounds of each scan frame in each axis"),
+    ]
 
     @resolver
     def smallest_step(self) -> float:
-        """The smallest step between midpoints in this axis"""
-        return calc_smallest_step([self.midpoints.get_points()])
+        """The smallest step between midpoints in this axis."""
+        return _calc_smallest_step([self.midpoints.get_points()])
 
 
 @dataclass
 class PointsResponse:
-    """ The highest level of the getPoints query, allowing users to customise their
-    return data from the points present in the scan to some metadata about them
-    """
+    """Information about the points provided by a spec."""
 
     total_frames: int
     returned_frames: int
 
-    def __init__(self, chunk: Dimension, total_frames: int):
+    def __init__(self, chunk: Frames[str], total_frames: int):
         self.total_frames = total_frames
         """The number of frames present across the entire spec"""
         self.returned_frames = len(chunk)
@@ -80,7 +81,7 @@ class PointsResponse:
 
     @resolver
     def axes(self) -> List[AxisFrames]:
-        """A list of all of the points present in the spec per axis"""
+        """A list of all of the points present in the spec per axis."""
         return [
             AxisFrames(
                 axis,
@@ -88,16 +89,16 @@ class PointsResponse:
                 Points(self._chunk.midpoints[axis]),
                 Points(self._chunk.upper[axis]),
             )
-            for axis in self._chunk.midpoints
+            for axis in self._chunk.axes()
         ]
 
     @resolver
     def smallest_abs_step(self) -> float:
-        """The smallest step between midpoints across ALL axes in the scan"""
-        return calc_smallest_step(list(self._chunk.midpoints.values()))
+        """The smallest step between midpoints across ALL axes in the scan."""
+        return _calc_smallest_step(list(self._chunk.midpoints.values()))
 
 
-def calc_smallest_step(points: List[np.ndarray]) -> float:
+def _calc_smallest_step(points: List[np.ndarray]) -> float:
     # Calc abs diffs of all axes
     absolute_diffs = [abs_diffs(axis_midpoints) for axis_midpoints in points]
     # Return the smallest value (Aka. smallest step)
@@ -105,14 +106,13 @@ def calc_smallest_step(points: List[np.ndarray]) -> float:
 
 
 def abs_diffs(array: np.ndarray) -> np.ndarray:
-    """Calculates the absolute differences between adjacent elements in the array
-    used as part of the smallest step calculation for each axis
+    """Calculates the absolute differences between adjacent elements in the array.
 
     Args:
-        array (ndarray): A 1xN array of numerical values
+        array: A 1xN array of numerical values
 
     Returns:
-        ndarray: A newly constucted array of absolute differences
+        A newly constucted array of absolute differences
     """
     # [array[1] - array[0], array[2] - array[1], ...]
     adjacent_diffs = array[1:] - array[:-1]
@@ -120,27 +120,21 @@ def abs_diffs(array: np.ndarray) -> np.ndarray:
 
 
 # Checks that the spec will produce a valid scan
-def validate_spec(spec: Spec) -> Any:
-    """ A query used to confirm whether or not the Spec will produce a viable scan"""
+def validate_spec(spec: Spec[str]) -> Any:
+    """A query used to confirm whether or not the Spec will produce a viable scan."""
     # apischema will do all the validation for us
     return spec.serialize()
 
 
 # Returns a full list of points for each axis in the scan
-# TODO Update max_frames with a more sophisticated method of reducing scan points
-def get_points(spec: Spec, max_frames: Optional[int] = 100000) -> PointsResponse:
-    """A query that takes a Spec and calculates the points present in the scan
-    (for each axis) plus some metadata about the points.
+def get_points(spec: Spec[str], max_frames: Optional[int] = 100000) -> PointsResponse:
+    """Calculate the frames present in the scan plus some metadata about the points.
 
-    Arguments:
-            [spec]: [The specification of the scan]
-            [max_frames]: [The maximum number of frames the user wishes to receive]
-
-    Returns:
-        [PointsResponse]: [A dataclass containing information about the scan points
-                            present in the spec]
+    Args:
+        spec: The specification of the scan
+        max_frames: The maximum number of frames the user wishes to receive
     """
-    dims = spec.create_dimensions()  # Grab dimensions from spec
+    dims = spec.calculate()  # Grab dimensions from spec
     path = Path(dims)  # Convert to a path
 
     # TOTAL FRAMES
@@ -158,54 +152,50 @@ def get_points(spec: Spec, max_frames: Optional[int] = 100000) -> PointsResponse
 
 
 # Define the schema
-schema = graphql_schema(query=[validate_spec, get_points])
+scanspec_schema = graphql_schema(query=[validate_spec, get_points])
 
 
-def reduce_frames(dims: List[Dimension], max_frames: int) -> Path:
-    """Removes frames from a spec such that it produces a number that is
-    closest to the max points value
+def reduce_frames(stack: List[Frames[str]], max_frames: int) -> Path:
+    """Removes frames from a spec so len(path) < max_frames.
 
     Args:
-        dims (List[Dimension]): A dimension object created by a spec
-        max_frames (int): The maximum number of frames the user wishes to be returned
-
-    Returns:
-        Path: A consumable object containing the expanded dimension with reduced frames
+        stack: A stack of Frames created by a spec
+        max_frames: The maximum number of frames the user wishes to be returned
     """
     # Calculate the total number of frames
     num_frames = 1
-    for dim in dims:
-        num_frames *= len(dim)
+    for frames in stack:
+        num_frames *= len(frames)
 
     # Need each dim to be this much smaller
-    ratio = 1 / np.power(max_frames / num_frames, 1 / len(dims))
+    ratio = 1 / np.power(max_frames / num_frames, 1 / len(stack))
 
-    sub_dims = [sub_sample(d, ratio) for d in dims]
-    return Path(sub_dims)
+    sub_frames = [sub_sample(f, ratio) for f in stack]
+    return Path(sub_frames)
 
 
-def sub_sample(dim: Dimension, ratio: float) -> Dimension:
-    """Removes frames from a dimension whilst preserving its core structure
+def sub_sample(frames: Frames[str], ratio: float) -> Frames:
+    """Provides a sub-sample Frames object whilst preserving its core structure.
 
     Args:
-        dim (Dimension): the dimension object to be reduced
-        ratio (float): the reduction ratio of the dimension
-    Returns:
-        Dimension: the reduced dimension
+        frames: the Frames object to be reduced
+        ratio: the reduction ratio of the dimension
     """
-    num_indexes = int(len(dim) / ratio)
-    indexes = np.linspace(0, len(dim) - 1, num_indexes).astype(np.int32)
-    return dim[indexes]
+    num_indexes = int(len(frames) / ratio)
+    indexes = np.linspace(0, len(frames) - 1, num_indexes, dtype=np.int32)
+    return frames.extract(indexes, calculate_gap=False)
 
 
-def schema_text() -> str:
-    return graphql.utilities.print_schema(schema)
+def scanspec_schema_text() -> str:
+    """Return the text representation of the GraphQL schema."""
+    return graphql.utilities.print_schema(scanspec_schema)
 
 
 def run_app(cors=False, port=8080):
+    """Run an application providing the scanspec service."""
     app = web.Application()
 
-    view = GraphQLView(schema=schema, graphiql=True)
+    view = GraphQLView(schema=scanspec_schema, graphiql=True)
 
     # Make GraphQLView compatible with aiohttp-cors
     # https://github.com/aio-libs/aiohttp-cors/issues/241#issuecomment-514278001
